float SSAO(vec3 viewPos, vec3 normal){
    float noise = rand2_1(texcoord + sin(frameTimeCounter));
    vec3 randomVec = rand2_3(texcoord + sin(frameTimeCounter)) * 2.0 - 1.0;

    vec3 tangent = normalize(randomVec - normal * dot(randomVec, normal));
    vec3 bitangent = normalize(cross(normal, tangent));
    mat3 TBN = mat3(tangent, bitangent, normal);

    float N_SAMPLES = remapSaturate(length(viewPos), 0.0, 120.0, SSAO_MAX_SAMPLES, SSAO_MIN_SAMPLES);
    const float radius = SSAO_SEARCH_RADIUS;

    float ao = 0.0;
    for(int i = 0; i < N_SAMPLES; ++i){
        vec3 offset = rand2_3(texcoord + sin(frameTimeCounter) + i);
        offset.xy = offset.xy * 2.0 - 1.0;
            float scale = float(i) / N_SAMPLES;
            scale = lerp(0.1, 1.0, scale * scale);
            offset *= scale;
        offset = TBN * offset;

        vec3 sampleViewPos = viewPos.xyz + offset * radius;
        vec3 sampleScreenPos = viewPosToScreenPos(vec4(sampleViewPos, 1.0)).xyz;
        float sampleDepth = texture(depthtex1, sampleScreenPos.xy).r;

        sampleDepth = linearizeDepth(sampleDepth);
        sampleScreenPos.z = linearizeDepth(sampleScreenPos.z);

        float nowAO = 0.0;
        if(sampleDepth < sampleScreenPos.z){
            float weight = 1.0;            

            float rangeCheck = smoothstep(0.0, 1.0, 1.0 / (sampleScreenPos.z - sampleDepth));
            weight *= rangeCheck;
            
            if(outScreen(sampleScreenPos.xy))
                weight = 0.0;

            nowAO = 1.0 * weight;
        }
        ao += (1.0 - nowAO);
    }

    ao /= N_SAMPLES;
    ao = pow(ao, SSAO_INTENSITY);
    return saturate(ao);
}



float HBAO(vec3 viewPos, vec3 normal){
    const int N_SAMPLES = 64;
    float dist = length(viewPos);
    float radius = 0.75;

    float ao = 0.0;
    for(int i = 0; i < N_SAMPLES; ++i){
        float rand1 = rand2_1(texcoord + sin(frameTimeCounter) + i);
        float rand2 = rand2_1(texcoord + sin(frameTimeCounter) + i + vec2(17.33333));
        float angle = rand2 * _2PI;
        vec2 offsetUV = vec2(rand1 * sin(angle), rand1 * cos(angle)) * radius / dist;

        vec2 curUV = texcoord + offsetUV;
            
        float z = texture(depthtex1, curUV).r;
        vec3 curViewPos = screenPosToViewPos(vec4(curUV, z, 1.0)).xyz;
        
        vec3 vector = curViewPos - viewPos + normalize(viewPos) * 0.1;

        float cosTheta = dot(normal, normalize(vector));
        // float sinTheta = sqrt(1.0 - cosTheta * cosTheta);
        // if(sign(cosTheta) < 0.0)
        //     sinTheta = 0.0;

        float weight = max(0.0, 1.0 - length(vector) / radius);

        if(outScreen(curUV)) weight = 0.0;

        ao += saturate(cosTheta) * weight;
    }
    ao /= N_SAMPLES;

    return saturate(1.0 - 10.0 * ao);
}



float getCosH(float i, float dir, vec3 p_c, vec3 v){
    vec2 offsetUV = vec2(cos(dir), sin(dir)) * i;
    vec2 curUV = texcoord + offsetUV;
    float z = texture(depthtex1, curUV).r;
    vec3 p_s = screenPosToViewPos(vec4(curUV, z, 1.0)).xyz;
    vec3 d_s = p_s - p_c;
    return dot(d_s, v) / length(d_s);
}

float GTAO(vec3 viewPos, vec3 normal){
    float rand = rand2_1(texcoord + sin(frameTimeCounter));
    float dist = length(viewPos);
    vec3 v = -normalize(viewPos);
    vec3 p_c = viewPos;

    const float N_DIR = 4.0;
    float dDir = PI / N_DIR;
    float startDir = rand * dDir;

    const float N_STEP = 6.0;
    float radius = 0.3 / dist;
    float dStep = radius / N_STEP;
    float startStep = rand * dStep;

    float ao = 0.0;
    for(float dir = startDir; dir < PI; dir += dDir){
        vec3 sliceDir = vec3(cos(dir), sin(dir), 0.0);
        vec3 crossSV = normalize(cross(sliceDir, v));
        vec3 projN = normal - crossSV * dot(normal, crossSV);
        float cosN = dot(normalize(projN), v);
        float n = acos(cosN);

        float h1 = 0.0;
        for(float i = startStep; i <= radius; i += dStep){
            float h = getCosH(i, dir, p_c, v);
            h1 = max(h1, h);
        }
        h1 = -acos(h1);

        float h2 = 0.0;
        for(float i = startStep; i <= radius; i += dStep){
            float h = getCosH(i, dir + PI, p_c, v);
            h2 = max(h2, h);
        }
        h2 = acos(h2);

        h1 = n + max(h1 - n, -PI / 2);
        h2 = n + min(h2 - n, PI / 2);

        float v_d = 0.25 * (-cos(2 * h1 - n) + cosN + 2 * h1 * sin(n))
                + 0.25 * (-cos(2 * h2 - n) + cosN + 2 * h2 * sin(n));
        v_d *= length(projN);
        ao += saturate(v_d);
    }
    ao /= N_DIR;

    return ao;
}

vec3 AOMultiBounce( vec3 BaseColor, float ao ){
	vec3 a =  2.0404 * BaseColor - 0.3324;
	vec3 b = -4.7951 * BaseColor + 0.6417;
	vec3 c =  2.7552 * BaseColor + 0.6903;
	return max( vec3(ao), ( ( ao * a + b ) * ao + c ) * ao );
}