varying vec2 texcoord;

varying vec3 sunWorldDir, moonWorldDir, lightWorldDir;
varying vec3 sunViewDir, moonViewDir, lightViewDir;

varying vec3 sunColor, skyColor;
varying vec3 zenithColor, horizonColor;

varying float isNoon, isNight, sunRiseSet;



#include "/lib/uniform.glsl"
#include "/lib/settings.glsl"
#include "/lib/common/utils.glsl"
#include "/lib/camera/colorToolkit.glsl"
#include "/lib/camera/filter.glsl"
#include "/lib/common/position.glsl"
#include "/lib/common/normal.glsl"
#include "/lib/common/noise.glsl"
#include "/lib/atmosphere/atmosphericScattering.glsl"

#ifdef FSH
const bool shadowtex1Mipmap = true;
const bool shadowtex1Nearest = false;

const bool shadowcolor0Mipmap = true;
const bool shadowcolor0Nearest = false;
const bool shadowcolor1Mipmap = true;
const bool shadowcolor1Nearest = false;

#include "/lib/common/gbufferData.glsl"
#include "/lib/common/materialIdMapper.glsl"
#include "/lib/lighting/RSM.glsl"
#include "/lib/lighting/SSAO.glsl"

void main() {
	vec4 CT1 = texelFetch(colortex1, ivec2(gl_FragCoord.xy), 0);
	vec4 CT3 = texelFetch(colortex3, ivec2(gl_FragCoord.xy), 0);
	vec4 CT6 = texelFetch(colortex6, ivec2(gl_FragCoord.xy), 0);

	vec2 hrrUV_a = texcoord * 2.0 - 1.0;
	if(!outScreen(hrrUV_a)){
		float hrrZ = texture(depthtex1, hrrUV_a).x;
		vec4 hrrScreenPos = vec4(unTAAJitter(hrrUV_a), hrrZ, 1.0);
		vec4 hrrViewPos = screenPosToViewPos(hrrScreenPos);
		vec4 hrrWorldPos = viewPosToWorldPos(hrrViewPos);
		vec3 hrrWorldDir = normalize(hrrWorldPos.xyz);

		if(isSkyHRR() > 0.5) {
			float d_p2a = RaySphereIntersection(earthPos, hrrWorldDir, vec3(0.0), earth_r + atmosphere_h).y;
			float d_p2e = RaySphereIntersection(earthPos, hrrWorldDir, vec3(0.0), earth_r).x;
			float d = d_p2e > 0.0 ? d_p2e : d_p2a;
			d = max(d, 0.0);

			mat2x3 atmosphericScattering = AtmosphericScattering(hrrWorldDir * d, sunWorldDir, IncomingLight, 1.0, ATMOSPHERE_SCATTERING_SAMPLES, ATMOSPHERE_LUT_SAMPLE_INTERVAL);
			atmosphericScattering += AtmosphericScattering(hrrWorldDir * d, moonWorldDir, IncomingLight_N, 1.0, int(ATMOSPHERE_SCATTERING_SAMPLES * 0.5), ATMOSPHERE_LUT_SAMPLE_INTERVAL * 4) * 0.002 * SKY_BASE_COLOR_BRIGHTNESS_N;
			CT1.rgb = atmosphericScattering[0] + atmosphericScattering[1];
		}
	}

	vec2 hrrUV = texcoord * 2.0;
	float hrrZ = CT6.a;
	vec3 rsm = BLACK;
	float ao = 1.0;
	if(!outScreen(hrrUV) && hrrZ < 1.0){
		vec4 hrrScreenPos = vec4(unTAAJitter(hrrUV), hrrZ, 1.0);
		vec4 hrrViewPos = screenPosToViewPos(hrrScreenPos);
		vec4 hrrWorldPos = viewPosToWorldPos(hrrViewPos);

		vec3 hrrNormalW = CT6.xyz;
		vec3 hrrNormal = normalize(gbufferModelView * vec4(hrrNormalW, 0.0)).xyz;

		#ifdef RSM_ENABLED
			if(isNoon > 0.0) rsm = RSM(hrrWorldPos, hrrNormalW);
			rsm = max(BLACK, rsm);
		#endif

		#ifdef SSAO_ENABLED
			ao = SSAO(hrrViewPos.xyz, hrrNormal);
		#endif

		vec4 gi = vec4(rsm, ao);
		#if defined RSM_ENABLED || defined SSAO_ENABLED
			gi = temporal_RSM(gi);
			gi = max(vec4(0.0), gi);
			CT1 = gi;
			CT3 = gi;
		#endif
	}

	if(ivec2(gl_FragCoord.xy) == SUN_COLOR_UV){
		CT1.rgb = sunColor;
	}

	if(ivec2(gl_FragCoord.xy) == SKY_COLOR_UV){
		CT1.rgb = skyColor;
	}

	if(ivec2(gl_FragCoord.xy) == ZENITH_COLOR_UV){
		CT1.rgb = zenithColor;
	}

	if(ivec2(gl_FragCoord.xy) == HORIZON_COLOR_UV){
		CT1.rgb = horizonColor;
	}


/* DRAWBUFFERS:13 */
	gl_FragData[0] = CT1;
	gl_FragData[1] = CT3;
}

#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH

void main() {
	sunViewDir = normalize(sunPosition);
	moonViewDir = normalize(moonPosition);
	lightViewDir = normalize(shadowLightPosition);

	sunWorldDir = normalize(viewPosToWorldPos(vec4(sunPosition, 0.0)).xyz);
    moonWorldDir = normalize(viewPosToWorldPos(vec4(moonPosition, 0.0)).xyz);
    lightWorldDir = normalize(viewPosToWorldPos(vec4(shadowLightPosition, 0.0)).xyz);

	isNoon = saturate(dot(sunWorldDir, upWorldDir) * NOON_DURATION);
	isNight = saturate(dot(moonWorldDir, upWorldDir) * NIGHT_DURATION);
	sunRiseSet = saturate(1 - isNoon - isNight);

	float d1 = RaySphereIntersection(earthPos, vec3(0.0, 1.0, 0.0), vec3(0.0), earth_r + atmosphere_h).y;
	// vec3 worldPos, vec3 lightDir, vec3 I, float mieAmount, const int N_SAMPLES, const int lutSampleGap
	mat2x3 atmosphericScattering = AtmosphericScattering(upWorldDir * d1, sunWorldDir, IncomingLight, 0.0, ATMOSPHERE_SCATTERING_SAMPLES, ATMOSPHERE_LUT_SAMPLE_INTERVAL);
	zenithColor = atmosphericScattering[0] + atmosphericScattering[1];

	vec3 horizonDir = normalize(vec3(0.0, 0.1, -1.0));
	float d2 = RaySphereIntersection(earthPos, horizonDir, vec3(0.0), earth_r + atmosphere_h).y;
	atmosphericScattering = AtmosphericScattering(horizonDir * d2, sunWorldDir, IncomingLight, 0.0, ATMOSPHERE_SCATTERING_SAMPLES, ATMOSPHERE_LUT_SAMPLE_INTERVAL);
	horizonColor = atmosphericScattering[0] + atmosphericScattering[1];

	sunColor = isNoon * TransmittanceToAtmosphere(earthPos, sunWorldDir) * IncomingLight;
	sunColor += isNight * TransmittanceToAtmosphere(earthPos, moonWorldDir) * IncomingLight_N;
	sunColor *= 1.0 - 0.75 * rainStrength;
	// skyColor = GetMultiScattering(getHeigth(earthPos), upWorldDir, sunWorldDir) * IncomingLight * 3;
	// skyColor += GetMultiScattering(getHeigth(earthPos), upWorldDir, moonWorldDir) * IncomingLight_N * 3;
	skyColor = zenithColor + isNight * 0.1 * IncomingLight_N / IncomingLight;
	skyColor *= 3.0;

	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
}

#endif