varying vec2 texcoord;

varying vec3 sunWorldDir, moonWorldDir, lightWorldDir;
varying vec3 sunViewDir, moonViewDir, lightViewDir;

varying vec3 sunColor, skyColor;
varying vec3 zenithColor, horizonColor;

varying float isNoon, isNight, sunRiseSet;



#include "/lib/uniform.glsl"
#include "/lib/settings.glsl"
#include "/lib/common/utils.glsl"
#include "/lib/camera/colorToolkit.glsl"
#include "/lib/camera/filter.glsl"
#include "/lib/common/position.glsl"
#include "/lib/common/normal.glsl"
#include "/lib/common/noise.glsl"
#include "/lib/atmosphere/atmosphericScattering.glsl"

#ifdef FSH
const bool shadowtex1Mipmap = true;
const bool shadowtex1Nearest = false;

const bool shadowcolor0Mipmap = true;
const bool shadowcolor0Nearest = false;
const bool shadowcolor1Mipmap = true;
const bool shadowcolor1Nearest = false;

#include "/lib/common/gbufferData.glsl"
#include "/lib/common/materialIdMapper.glsl"
#include "/lib/lighting/RSM.glsl"
#include "/lib/lighting/SSAO.glsl"

void main() {
	vec4 CT1 = texture(colortex1, texcoord);
	vec4 CT3 = texture(colortex3, texcoord);

	vec2 hrrUV = texcoord * 2.0;
	float hrrZ = texture(depthtex1, hrrUV).x;
	vec3 rsm = BLACK;
	float ao = 1.0;
	if(!outScreen(hrrUV) && hrrZ < 1.0){
		vec4 hrrScreenPos = vec4(unTAAJitter(hrrUV), hrrZ, 1.0);
		vec4 hrrViewPos = screenPosToViewPos(hrrScreenPos);
		vec4 hrrWorldPos = viewPosToWorldPos(hrrViewPos);

		vec3 hrrNormal = getNormal(hrrUV);
		vec3 hrrNormalW = normalize(viewPosToWorldPos(vec4(hrrNormal, 0.0)).xyz);

		#ifdef SSAO_ENABLED
			ao = SSAO(hrrViewPos.xyz, hrrNormal);
		#endif

		vec4 gi = vec4(rsm, ao);
		#if defined RSM_ENABLED || defined SSAO_ENABLED
			gi = temporal_RSM(gi);
			gi = max(vec4(0.0), gi);
			CT1 = gi;
			CT3 = gi;
		#endif
	}

	if(ivec2(gl_FragCoord.xy) == SUN_COLOR_UV){
		CT1.rgb = sunColor;
	}

	if(ivec2(gl_FragCoord.xy) == SKY_COLOR_UV){
		CT1.rgb = skyColor;
	}

	if(ivec2(gl_FragCoord.xy) == ZENITH_COLOR_UV){
		CT1.rgb = zenithColor;
	}

	if(ivec2(gl_FragCoord.xy) == HORIZON_COLOR_UV){
		CT1.rgb = horizonColor;
	}


/* DRAWBUFFERS:13 */
	gl_FragData[0] = CT1;
	gl_FragData[1] = CT3;
}

#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH

void main() {
	sunViewDir = normalize(sunPosition);
	moonViewDir = normalize(moonPosition);
	lightViewDir = normalize(shadowLightPosition);

	sunWorldDir = normalize(viewPosToWorldPos(vec4(sunPosition, 0.0)).xyz);
    moonWorldDir = normalize(viewPosToWorldPos(vec4(moonPosition, 0.0)).xyz);
    lightWorldDir = normalize(viewPosToWorldPos(vec4(shadowLightPosition, 0.0)).xyz);

	isNoon = saturate(dot(sunWorldDir, upWorldDir) * NOON_DURATION);
	isNight = saturate(dot(moonWorldDir, upWorldDir) * NIGHT_DURATION);
	sunRiseSet = saturate(1 - isNoon - isNight);

	sunColor = isNoon * TransmittanceToAtmosphere(earthPos, sunWorldDir) * IncomingLight;
	sunColor += isNight * TransmittanceToAtmosphere(earthPos, moonWorldDir) * IncomingLight_N;
	sunColor *= 1.0 - 0.75 * rainStrength;
	skyColor = GetMultiScattering(getHeigth(earthPos), upWorldDir, sunWorldDir) * IncomingLight * 3;
	skyColor += GetMultiScattering(getHeigth(earthPos), upWorldDir, moonWorldDir) * IncomingLight_N * 3;

	float d1 = RaySphereIntersection(earthPos, upWorldDir, vec3(0.0), earth_r + atmosphere_h).y;
	mat2x3 atmosphericScattering = AtmosphericScattering(upWorldDir * d1, sunWorldDir, IncomingLight, 0.0, ATMOSPHERE_SCATTERING_SAMPLES, ATMOSPHERE_LUT_SAMPLE_INTERVAL);
	zenithColor = atmosphericScattering[0] + atmosphericScattering[1];

	vec3 horizonDir = normalize(vec3(0.0, 0.05, -1.0));
	float d2 = RaySphereIntersection(earthPos, horizonDir, vec3(0.0), earth_r + atmosphere_h).y;
	atmosphericScattering = AtmosphericScattering(horizonDir * d2, sunWorldDir, IncomingLight, 0.0, ATMOSPHERE_SCATTERING_SAMPLES, ATMOSPHERE_LUT_SAMPLE_INTERVAL);
	horizonColor = atmosphericScattering[0] + atmosphericScattering[1];

	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
}

#endif